@@ -419,21 +419,18 @@ zmq::thread_ctx_t::thread_ctx_t () :
 
 void zmq::thread_ctx_t::start_thread (thread_t &thread_,
                                       thread_fn *tfn_,
-                                      void *arg_) const
+                                      void *arg_,
+                                      const char *name_) const
 {
-    static unsigned int nthreads_started = 0;
-
     thread_.setSchedulingParameters (_thread_priority, _thread_sched_policy,
                                      _thread_affinity_cpus);
-    thread_.start (tfn_, arg_);
-#ifndef ZMQ_HAVE_ANDROID
-    std::ostringstream s;
-    if (!_thread_name_prefix.empty ())
-        s << _thread_name_prefix << "/";
-    s << "ZMQbg/" << nthreads_started;
-    thread_.setThreadName (s.str ().c_str ());
-#endif
-    nthreads_started++;
+
+    char namebuf[16] = "";
+    snprintf (namebuf, sizeof (namebuf), "%s%sZMQbg%s%s",
+              _thread_name_prefix.empty () ? "" : _thread_name_prefix.c_str (),
+              _thread_name_prefix.empty () ? "" : "/", name_ ? "/" : "",
+              name_ ? name_ : "");
+    thread_.start (tfn_, arg_, namebuf);
 }
 
 int zmq::thread_ctx_t::set (int option_, int optval_)
@@ -469,10 +466,7 @@ int zmq::thread_ctx_t::set (int option_, int optval_)
 int zmq::thread_ctx_t::get (int option_)
 {
     int rc = 0;
-    if (option_ == ZMQ_THREAD_PRIORITY) {
-        scoped_lock_t locker (_opt_sync);
-        rc = _thread_priority;
-    } else if (option_ == ZMQ_THREAD_SCHED_POLICY) {
+    if (option_ == ZMQ_THREAD_SCHED_POLICY) {
         scoped_lock_t locker (_opt_sync);
         rc = _thread_sched_policy;
     } else if (option_ == ZMQ_THREAD_NAME_PREFIX) {
